@@ -8,6 +8,8 @@ from .utils import (
     validate,
     abs_path,
     monitor,
+    clean_state,
+    base_directory,
 )
 from .backends import create_backend
 
@@ -20,14 +22,13 @@ class App:
     def __init__(
         self,
         name,
-        root=None,
         backend=False,
         debug=False,
         create_protocols=None,
         **kwargs,
     ):
         self.name = name
-        self._root = root
+        self._root = base_directory()
         self._backend = create_backend(self, backend, create_protocols)
         self._debug = debug
 
@@ -107,9 +108,22 @@ class App:
             return self._parser
 
         self._parser = argparse.ArgumentParser(description="PyWebVue server By Kitware")
-        self._parser.add_argument("--launcher",
+
+        # Launcher / multi-users
+        self._parser.add_argument(
+            "--launcher",
             help="Start launcher process rather than single process server",
-            action="store_true")
+            action="store_true",
+        )
+
+        # Deploy helper
+        self._parser.add_argument("--deploy", help="Prepare a deployable directory")
+        self._parser.add_argument(
+            "--name",
+            default="pywebvue",
+            help="Name to use for that application in the deploy tree",
+        )
+
         CoreServer.add_arguments(self._parser)
 
         return self._parser
@@ -166,8 +180,8 @@ class App:
 
     # -------------------------------------------------------------------------
 
-    def set(self, key, value):
-        if key not in self.state or self.state[key] != value:
+    def set(self, key, value, force=False):
+        if key not in self.state or self.state[key] != value or force:
             self.state[key] = value
             for change_handler in self._change_handlers:
                 change_handler.modified(key, value)
@@ -255,7 +269,7 @@ class App:
             "layout": self.layout,
             "favicon": self.favicon,
             "routes": self.routes,
-            "state": self.state,
+            "state": clean_state(self.state),
             "scripts": self.scripts,
             "use": self.vue_use,
             "styles": self.styles,
@@ -297,6 +311,8 @@ class App:
         if args.launcher:
             CoreServer.start_launcher(args)
             # FIXME we don't support yet self.serve with launcher...
+        elif args.deploy:
+            CoreServer.deploy_setup(args)
         else:
             if len(self.serve):
                 endpoints = []
